@@ -99,7 +99,12 @@ namespace {
             return m_ns.get();
         }
 
-        XMLObject* decode(string& relayState, const GenericRequest& genericRequest, SecurityPolicy& policy) const;
+        XMLObject* decode(
+            string& relayState,
+            const GenericRequest& genericRequest,
+            const GenericResponse* genericResponse,
+            SecurityPolicy& policy
+        ) const;
 
     protected:
         void extractMessageDetails(
@@ -189,6 +194,10 @@ namespace {
         }
 
     private:
+        const char* getProfile() const {
+            return WSFED_NS;
+        }
+
         void implementProtocol(
             const Application& application,
             const HTTPRequest& httpRequest,
@@ -473,7 +482,7 @@ void ADFSSessionInitiator::receive(DDF& in, ostream& out)
     DDFJanitor jout(ret);
 
     // Wrap the outgoing object with a Response facade.
-    scoped_ptr<HTTPResponse> http(getResponse(ret));
+    scoped_ptr<HTTPResponse> http(getResponse(*app, ret));
 
     string relayState(in["RelayState"].string() ? in["RelayState"].string() : "");
 
@@ -574,7 +583,9 @@ pair<bool,long> ADFSSessionInitiator::doRequest(
 
 #ifndef SHIBSP_LITE
 
-XMLObject* ADFSDecoder::decode(string& relayState, const GenericRequest& genericRequest, SecurityPolicy& policy) const
+XMLObject* ADFSDecoder::decode(
+    string& relayState, const GenericRequest& genericRequest, const GenericResponse*, SecurityPolicy& policy
+    ) const
 {
 #ifdef _DEBUG
     xmltooling::NDC ndc("decode");
@@ -836,7 +847,7 @@ void ADFSConsumer::implementProtocol(
 
 #endif
 
-pair<bool,long> ADFSLogoutInitiator::run(SPRequest& request, bool isHandler) const
+pair<bool,long> ADFSLogoutInitiator::run(SPRequest& request, bool) const
 {
     // Normally we'd do notifications and session clearage here, but ADFS logout
     // is missing the needed request/response features, so we have to rely on
@@ -894,12 +905,12 @@ void ADFSLogoutInitiator::receive(DDF& in, ostream& out)
     }
 
     // Unpack the request.
-    scoped_ptr<HTTPRequest> req(getRequest(in));
+    scoped_ptr<HTTPRequest> req(getRequest(*app, in));
 
     // Set up a response shim.
     DDF ret(nullptr);
     DDFJanitor jout(ret);
-    scoped_ptr<HTTPResponse> resp(getResponse(ret));
+    scoped_ptr<HTTPResponse> resp(getResponse(*app, ret));
 
     Session* session = nullptr;
     try {
