@@ -71,7 +71,7 @@ namespace shibsp {
     class SHIBSP_DLLLOCAL XMLFilterImpl
     {
     public:
-        XMLFilterImpl(const DOMElement* e, Category& log);
+        XMLFilterImpl(const DOMElement* e, Category& log, bool deprecationSupport);
         ~XMLFilterImpl() {
             if (m_document)
                 m_document->release();
@@ -88,16 +88,16 @@ namespace shibsp {
 
     private:
         MatchFunctor* buildFunctor(
-            const DOMElement* e, const FilterPolicyContext& functorMap, const char* logname, bool standalone
+            const DOMElement* e, const FilterPolicyContext& functorMap, const char* logname, bool standalone, bool deprecationSupport
             );
-        tuple<string,const MatchFunctor*,const MatchFunctor*> buildAttributeRule(
-            const DOMElement* e, const FilterPolicyContext& permMap, const FilterPolicyContext& denyMap, bool standalone
+        boost::tuple<string,const MatchFunctor*,const MatchFunctor*> buildAttributeRule(
+            const DOMElement* e, const FilterPolicyContext& permMap, const FilterPolicyContext& denyMap, bool standalone, bool deprecationSupport
             );
 
         Category& m_log;
         DOMDocument* m_document;
         vector<Policy> m_policies;
-        map< string,tuple<string,const MatchFunctor*,const MatchFunctor*> > m_attrRules;
+        map< string,boost::tuple<string,const MatchFunctor*,const MatchFunctor*> > m_attrRules;
         multimap<string,MatchFunctor*> m_policyReqRules;
         multimap<string,MatchFunctor*> m_permitValRules;
         multimap<string,MatchFunctor*> m_denyValRules;
@@ -106,7 +106,8 @@ namespace shibsp {
     class SHIBSP_DLLLOCAL XMLFilter : public AttributeFilter, public ReloadableXMLFile
     {
     public:
-        XMLFilter(const DOMElement* e) : ReloadableXMLFile(e, Category::getInstance(SHIBSP_LOGCAT".AttributeFilter")) {
+        XMLFilter(const DOMElement* e, bool deprecationSupport=true)
+            : ReloadableXMLFile(e, Category::getInstance(SHIBSP_LOGCAT ".AttributeFilter"), true, deprecationSupport), m_deprecationSupport(deprecationSupport) {
             background_load();
         }
         ~XMLFilter() {
@@ -121,6 +122,7 @@ namespace shibsp {
         pair<bool,DOMElement*> background_load();
 
     private:
+        bool m_deprecationSupport;
         scoped_ptr<XMLFilterImpl> m_impl;
     };
 
@@ -128,7 +130,7 @@ namespace shibsp {
     #pragma warning( pop )
 #endif
 
-    AttributeFilter* SHIBSP_DLLLOCAL XMLAttributeFilterFactory(const DOMElement* const & e)
+    AttributeFilter* SHIBSP_DLLLOCAL XMLAttributeFilterFactory(const DOMElement* const & e, bool)
     {
         return new XMLFilter(e);
     }
@@ -144,11 +146,12 @@ namespace shibsp {
     static const XMLCh PolicyRequirementRule[] =        UNICODE_LITERAL_21(P,o,l,i,c,y,R,e,q,u,i,r,e,m,e,n,t,R,u,l,e);
     static const XMLCh PolicyRequirementRuleReference[]=UNICODE_LITERAL_30(P,o,l,i,c,y,R,e,q,u,i,r,e,m,e,n,t,R,u,l,e,R,e,f,e,r,e,n,c,e);
     static const XMLCh attributeID[] =                  UNICODE_LITERAL_11(a,t,t,r,i,b,u,t,e,I,D);
+    static const XMLCh permitAny[] =                    UNICODE_LITERAL_9(p,e,r,m,i,t,A,n,y);
     static const XMLCh _id[] =                          UNICODE_LITERAL_2(i,d);
     static const XMLCh _ref[] =                         UNICODE_LITERAL_3(r,e,f);
 };
 
-XMLFilterImpl::XMLFilterImpl(const DOMElement* e, Category& log) : m_log(log), m_document(nullptr)
+XMLFilterImpl::XMLFilterImpl(const DOMElement* e, Category& log, bool deprecationSupport) : m_log(log), m_document(nullptr)
 {
 #ifdef _DEBUG
     xmltooling::NDC ndc("XMLFilterImpl");
@@ -164,22 +167,22 @@ XMLFilterImpl::XMLFilterImpl(const DOMElement* e, Category& log) : m_log(log), m
     DOMElement* child = XMLHelper::getFirstChildElement(e);
     while (child) {
         if (XMLHelper::isNodeNamed(child, SHIB2ATTRIBUTEFILTER_NS, PolicyRequirementRule)) {
-            buildFunctor(child, reqFunctors, "PolicyRequirementRule", true);
+            buildFunctor(child, reqFunctors, "PolicyRequirementRule", true, deprecationSupport);
         }
         else if (XMLHelper::isNodeNamed(child, SHIB2ATTRIBUTEFILTER_NS, PermitValueRule)) {
-            buildFunctor(child, permFunctors, "PermitValueRule", true);
+            buildFunctor(child, permFunctors, "PermitValueRule", true, deprecationSupport);
         }
         else if (XMLHelper::isNodeNamed(child, SHIB2ATTRIBUTEFILTER_NS, DenyValueRule)) {
-            buildFunctor(child, denyFunctors, "DenyValueRule", true);
+            buildFunctor(child, denyFunctors, "DenyValueRule", true, deprecationSupport);
         }
         else if (XMLHelper::isNodeNamed(child, SHIB2ATTRIBUTEFILTER_NS, AttributeRule)) {
-            buildAttributeRule(child, permFunctors, denyFunctors, true);
+            buildAttributeRule(child, permFunctors, denyFunctors, true, deprecationSupport);
         }
         else if (XMLHelper::isNodeNamed(child, SHIB2ATTRIBUTEFILTER_NS, AttributeFilterPolicy)) {
             e = XMLHelper::getFirstChildElement(child);
             MatchFunctor* func = nullptr;
             if (e && XMLHelper::isNodeNamed(e, SHIB2ATTRIBUTEFILTER_NS, PolicyRequirementRule)) {
-                func = buildFunctor(e, reqFunctors, "PolicyRequirementRule", false);
+                func = buildFunctor(e, reqFunctors, "PolicyRequirementRule", false, deprecationSupport);
             }
             else if (e && XMLHelper::isNodeNamed(e, SHIB2ATTRIBUTEFILTER_NS, PolicyRequirementRuleReference)) {
                 string ref(XMLHelper::getAttrString(e, nullptr, _ref));
@@ -194,14 +197,14 @@ XMLFilterImpl::XMLFilterImpl(const DOMElement* e, Category& log) : m_log(log), m
                 e = XMLHelper::getNextSiblingElement(e);
                 while (e) {
                     if (e && XMLHelper::isNodeNamed(e, SHIB2ATTRIBUTEFILTER_NS, AttributeRule)) {
-                        tuple<string,const MatchFunctor*,const MatchFunctor*> rule = buildAttributeRule(e, permFunctors, denyFunctors, false);
+                        boost::tuple<string,const MatchFunctor*,const MatchFunctor*> rule = buildAttributeRule(e, permFunctors, denyFunctors, false, deprecationSupport);
                         if (rule.get<1>() || rule.get<2>())
                             m_policies.back().m_rules.insert(Policy::rules_t::value_type(rule.get<0>(), make_pair(rule.get<1>(), rule.get<2>())));
                     }
                     else if (e && XMLHelper::isNodeNamed(e, SHIB2ATTRIBUTEFILTER_NS, AttributeRuleReference)) {
                         string ref(XMLHelper::getAttrString(e, nullptr, _ref));
                         if (!ref.empty()) {
-                            map< string,tuple<string,const MatchFunctor*,const MatchFunctor*> >::const_iterator ar = m_attrRules.find(ref);
+                            map< string,boost::tuple<string,const MatchFunctor*,const MatchFunctor*> >::const_iterator ar = m_attrRules.find(ref);
                             if (ar != m_attrRules.end()) {
                                 m_policies.back().m_rules.insert(
                                     Policy::rules_t::value_type(ar->second.get<0>(), make_pair(ar->second.get<1>(), ar->second.get<2>()))
@@ -224,7 +227,7 @@ XMLFilterImpl::XMLFilterImpl(const DOMElement* e, Category& log) : m_log(log), m
 }
 
 MatchFunctor* XMLFilterImpl::buildFunctor(
-    const DOMElement* e, const FilterPolicyContext& functorMap, const char* logname, bool standalone
+    const DOMElement* e, const FilterPolicyContext& functorMap, const char* logname, bool standalone, bool deprecationSupport
     )
 {
     string id(XMLHelper::getAttrString(e, nullptr, _id));
@@ -244,12 +247,19 @@ MatchFunctor* XMLFilterImpl::buildFunctor(
 
     scoped_ptr<xmltooling::QName> type(XMLHelper::getXSIType(e));
     if (type) {
+        if (XMLString::equals(type->getNamespaceURI(), shibspconstants::SHIB2ATTRIBUTEFILTER_MF_BASIC_NS) ||
+            XMLString::equals(type->getNamespaceURI(), shibspconstants::SHIB2ATTRIBUTEFILTER_MF_SAML_NS)) {
+
+            auto_ptr_char ns(type->getNamespaceURI());
+            m_log.warn("Legacy filter namespace '%s' is DEPRECATED and will be removed from a future version.", ns.get());
+        }
+
         try {
-            auto_ptr<MatchFunctor> func(SPConfig::getConfig().MatchFunctorManager.newPlugin(*type, make_pair(&functorMap,e)));
+            auto_ptr<MatchFunctor> func(SPConfig::getConfig().MatchFunctorManager.newPlugin(*type, make_pair(&functorMap,e), deprecationSupport));
             functorMap.getMatchFunctors().insert(multimap<string,MatchFunctor*>::value_type(id, func.get()));
             return func.release();
         }
-        catch (exception& ex) {
+        catch (const exception& ex) {
             m_log.error("error building %s with type (%s): %s", logname, type->toString().c_str(), ex.what());
         }
     }
@@ -261,20 +271,20 @@ MatchFunctor* XMLFilterImpl::buildFunctor(
     return nullptr;
 }
 
-tuple<string,const MatchFunctor*,const MatchFunctor*> XMLFilterImpl::buildAttributeRule(
-    const DOMElement* e, const FilterPolicyContext& permMap, const FilterPolicyContext& denyMap, bool standalone
+boost::tuple<string,const MatchFunctor*,const MatchFunctor*> XMLFilterImpl::buildAttributeRule(
+    const DOMElement* e, const FilterPolicyContext& permMap, const FilterPolicyContext& denyMap, bool standalone, bool deprecationSupport
     )
 {
     string id(XMLHelper::getAttrString(e, nullptr, _id));
 
     if (standalone && id.empty()) {
         m_log.warn("skipping stand-alone AttributeRule with no id");
-        return tuple<string,const MatchFunctor*,const MatchFunctor*>(string(),nullptr,nullptr);
+        return boost::tuple<string,const MatchFunctor*,const MatchFunctor*>(string(),nullptr,nullptr);
     }
     else if (!id.empty() && m_attrRules.count(id)) {
         if (standalone) {
             m_log.warn("skipping duplicate stand-alone AttributeRule with id (%s)", id.c_str());
-            return tuple<string,const MatchFunctor*,const MatchFunctor*>(string(),nullptr,nullptr);
+            return boost::tuple<string,const MatchFunctor*,const MatchFunctor*>(string(),nullptr,nullptr);
         }
         else
             id.clear();
@@ -287,9 +297,15 @@ tuple<string,const MatchFunctor*,const MatchFunctor*> XMLFilterImpl::buildAttrib
     MatchFunctor* perm = nullptr;
     MatchFunctor* deny = nullptr;
 
+    if (XMLHelper::getAttrBool(e, false, permitAny)) {
+        m_log.debug("installing implicit ANY permit rule for attribute (%s)", attrID.c_str());
+        perm = SPConfig::getConfig().MatchFunctorManager.newPlugin(AnyMatchFunctorType, make_pair(&permMap,(const DOMElement*)nullptr), deprecationSupport);
+        return boost::make_tuple(attrID, perm, deny);
+    }
+
     e = XMLHelper::getFirstChildElement(e);
     if (e && XMLHelper::isNodeNamed(e, SHIB2ATTRIBUTEFILTER_NS, PermitValueRule)) {
-        perm = buildFunctor(e, permMap, "PermitValueRule", false);
+        perm = buildFunctor(e, permMap, "PermitValueRule", false, deprecationSupport);
         e = XMLHelper::getNextSiblingElement(e);
     }
     else if (e && XMLHelper::isNodeNamed(e, SHIB2ATTRIBUTEFILTER_NS, PermitValueRuleReference)) {
@@ -302,7 +318,7 @@ tuple<string,const MatchFunctor*,const MatchFunctor*> XMLFilterImpl::buildAttrib
     }
 
     if (e && XMLHelper::isNodeNamed(e, SHIB2ATTRIBUTEFILTER_NS, DenyValueRule)) {
-        deny = buildFunctor(e, denyMap, "DenyValueRule", false);
+        deny = buildFunctor(e, denyMap, "DenyValueRule", false, deprecationSupport);
     }
     else if (e && XMLHelper::isNodeNamed(e, SHIB2ATTRIBUTEFILTER_NS, DenyValueRuleReference)) {
         string ref(XMLHelper::getAttrString(e, nullptr, _ref));
@@ -314,11 +330,11 @@ tuple<string,const MatchFunctor*,const MatchFunctor*> XMLFilterImpl::buildAttrib
 
     if (perm || deny) {
         if (!id.empty()) {
-            m_attrRules[id] = make_tuple(attrID, perm, deny);
+            m_attrRules[id] = boost::make_tuple(attrID, perm, deny);
             return m_attrRules[id];
         }
         else {
-            return make_tuple(attrID, perm, deny);
+            return boost::make_tuple(attrID, perm, deny);
         }
     }
 
@@ -326,7 +342,7 @@ tuple<string,const MatchFunctor*,const MatchFunctor*> XMLFilterImpl::buildAttrib
         m_log.warn("skipping AttributeRule (%s), permit and denial rule(s) invalid or missing", id.c_str());
     else
         m_log.warn("skipping AttributeRule, permit and denial rule(s) invalid or missing");
-    return tuple<string,const MatchFunctor*,const MatchFunctor*>(string(),nullptr,nullptr);
+    return boost::tuple<string,const MatchFunctor*,const MatchFunctor*>(string(),nullptr,nullptr);
 }
 
 void XMLFilterImpl::filterAttributes(const FilteringContext& context, vector<Attribute*>& attributes) const
@@ -482,7 +498,7 @@ pair<bool,DOMElement*> XMLFilter::background_load()
     // If we own it, wrap it.
     XercesJanitor<DOMDocument> docjanitor(raw.first ? raw.second->getOwnerDocument() : nullptr);
 
-    scoped_ptr<XMLFilterImpl> impl(new XMLFilterImpl(raw.second, m_log));
+    scoped_ptr<XMLFilterImpl> impl(new XMLFilterImpl(raw.second, m_log, m_deprecationSupport));
 
     // If we held the document, transfer it to the impl. If we didn't, it's a no-op.
     impl->setDocument(docjanitor.release());
