@@ -68,7 +68,7 @@ namespace shibsp {
     class SHIBSP_DLLLOCAL SAML2LogoutInitiator : public AbstractHandler, public LogoutInitiator
     {
     public:
-        SAML2LogoutInitiator(const DOMElement* e, const char* appId);
+        SAML2LogoutInitiator(const DOMElement* e, const char* appId, bool deprecationSupport=true);
         virtual ~SAML2LogoutInitiator() {}
 
         void init(const char* location);    // encapsulates actions that need to run either in the c'tor or setParent
@@ -87,10 +87,15 @@ namespace shibsp {
             ) const;
 
         string m_appId;
+        bool m_deprecationSupport;
         auto_ptr_char m_protocol;
 #ifndef SHIBSP_LITE
         auto_ptr<LogoutRequest> buildRequest(
-            const Application& application, const Session& session, const RoleDescriptor& role, const MessageEncoder* encoder=nullptr
+            const Application& application,
+            const Session& session,
+            const RoleDescriptor& role,
+            const XMLCh* endpoint,
+            const MessageEncoder* encoder=nullptr
             ) const;
 
         LogoutEvent* newLogoutEvent(
@@ -112,14 +117,15 @@ namespace shibsp {
     #pragma warning( pop )
 #endif
 
-    Handler* SHIBSP_DLLLOCAL SAML2LogoutInitiatorFactory(const pair<const DOMElement*,const char*>& p)
+    Handler* SHIBSP_DLLLOCAL SAML2LogoutInitiatorFactory(const pair<const DOMElement*,const char*>& p, bool deprecationSupport)
     {
-        return new SAML2LogoutInitiator(p.first, p.second);
+        return new SAML2LogoutInitiator(p.first, p.second, deprecationSupport);
     }
 };
 
-SAML2LogoutInitiator::SAML2LogoutInitiator(const DOMElement* e, const char* appId)
-    : AbstractHandler(e, Category::getInstance(SHIBSP_LOGCAT".LogoutInitiator.SAML2")), m_appId(appId), m_protocol(samlconstants::SAML20P_NS)
+SAML2LogoutInitiator::SAML2LogoutInitiator(const DOMElement* e, const char* appId, bool deprecationSupport)
+    : AbstractHandler(e, Category::getInstance(SHIBSP_LOGCAT ".LogoutInitiator.SAML2")),
+        m_appId(appId), m_deprecationSupport(deprecationSupport), m_protocol(samlconstants::SAML20P_NS)
 #ifndef SHIBSP_LITE
         ,m_async(true)
 #endif
@@ -157,6 +163,7 @@ void SAML2LogoutInitiator::init(const char* location)
         pair<bool,const char*> outgoing = getString("outgoingBindings");
         if (outgoing.first) {
             dupBindings = outgoing.second;
+            trim(dupBindings);
         }
         else {
             // No override, so we'll install a default binding precedence.
@@ -166,9 +173,7 @@ void SAML2LogoutInitiator::init(const char* location)
         split(m_bindings, dupBindings, is_space(), algorithm::token_compress_on);
         for (vector<string>::const_iterator b = m_bindings.begin(); b != m_bindings.end(); ++b) {
             try {
-                boost::shared_ptr<MessageEncoder> encoder(
-                    SAMLConfig::getConfig().MessageEncoderManager.newPlugin(*b, pair<const DOMElement*,const XMLCh*>(getElement(),nullptr))
-                    );
+                boost::shared_ptr<MessageEncoder> encoder(SAMLConfig::getConfig().MessageEncoderManager.newPlugin(*b, getElement(), m_deprecationSupport));
                 if (encoder->isUserAgentPresent() && XMLString::equals(getProtocolFamily(), encoder->getProtocolFamily())) {
                     m_encoders[*b] = encoder;
                     m_log.debug("supporting outgoing binding (%s)", b->c_str());
@@ -177,7 +182,7 @@ void SAML2LogoutInitiator::init(const char* location)
                     m_log.warn("skipping outgoing binding (%s), not a SAML 2.0 front-channel mechanism", b->c_str());
                 }
             }
-            catch (std::exception& ex) {
+            catch (const std::exception& ex) {
                 m_log.error("error building MessageEncoder: %s", ex.what());
             }
         }
@@ -208,7 +213,7 @@ pair<bool,long> SAML2LogoutInitiator::run(SPRequest& request, bool isHandler) co
             return make_pair(false, 0L);
         }
     }
-    catch (std::exception& ex) {
+    catch (const std::exception& ex) {
         m_log.error("error accessing current session: %s", ex.what());
         return make_pair(false, 0L);
     }
@@ -223,7 +228,7 @@ pair<bool,long> SAML2LogoutInitiator::run(SPRequest& request, bool isHandler) co
         vector<string> headers(1,"Cookie");
         DDF out,in = wrap(request,&headers);
         DDFJanitor jin(in), jout(out);
-        out=request.getServiceProvider().getListenerService()->send(in);
+        out = send(request, in);
         return unwrap(request, out);
     }
 }
@@ -245,12 +250,12 @@ void SAML2LogoutInitiator::receive(DDF& in, ostream& out)
     }
 
     // Unpack the request.
-    scoped_ptr<HTTPRequest> req(getRequest(in));
+    scoped_ptr<HTTPRequest> req(getRequest(*app, in));
 
     // Set up a response shim.
     DDF ret(nullptr);
     DDFJanitor jout(ret);
-    scoped_ptr<HTTPResponse> resp(getResponse(ret));
+    scoped_ptr<HTTPResponse> resp(getResponse(*app, ret));
 
     Session* session = nullptr;
     try {
@@ -269,9 +274,10 @@ void SAML2LogoutInitiator::receive(DDF& in, ostream& out)
             doRequest(*app, *req, *resp, session);
         }
         else {
+            time_t revocationExp = session->getExpiration();
             session->unlock();
             m_log.log(getParent() ? Priority::WARN : Priority::ERROR, "bypassing SAML 2.0 logout, no NameID or issuing entityID found in session");
-            app->getServiceProvider().getSessionCache()->remove(*app, *req, resp.get());
+            app->getServiceProvider().getSessionCache()->remove(*app, *req, resp.get(), revocationExp);
         }
     }
     out << ret;
@@ -298,9 +304,10 @@ pair<bool,long> SAML2LogoutInitiator::doRequest(
             application.getServiceProvider().getTransactionLog()->write(*logout_event);
         }
 #endif
+        time_t revocationExp = session->getExpiration();
         sessionLocker.assign();
         session = nullptr;
-        application.getServiceProvider().getSessionCache()->remove(application, httpRequest, &httpResponse);
+        application.getServiceProvider().getSessionCache()->remove(application, httpRequest, &httpResponse, revocationExp);
         return sendLogoutPage(application, httpRequest, httpResponse, "partial");
     }
 
@@ -334,7 +341,8 @@ pair<bool,long> SAML2LogoutInitiator::doRequest(
         const MessageEncoder* encoder = nullptr;
         for (vector<string>::const_iterator b = m_bindings.begin(); b != m_bindings.end(); ++b) {
             auto_ptr_XMLCh wideb(b->c_str());
-            if (ep = EndpointManager<SingleLogoutService>(role->getSingleLogoutServices()).getByBinding(wideb.get())) {
+            ep = EndpointManager<SingleLogoutService>(role->getSingleLogoutServices()).getByBinding(wideb.get());
+            if (ep) {
                 map< string,boost::shared_ptr<MessageEncoder> >::const_iterator enc = m_encoders.find(*b);
                 if (enc != m_encoders.end())
                     encoder = enc->second.get();
@@ -356,7 +364,7 @@ pair<bool,long> SAML2LogoutInitiator::doRequest(
                 try {
                     if (!XMLString::equals(epit->getBinding(), binding.get()))
                         continue;
-                    auto_ptr<LogoutRequest> msg(buildRequest(application, *session, *role));
+                    auto_ptr<LogoutRequest> msg(buildRequest(application, *session, *role, epit->getLocation()));
 
                     // Log the request.
                     if (logout_event) {
@@ -366,15 +374,15 @@ pair<bool,long> SAML2LogoutInitiator::doRequest(
                         logout_event->m_saml2Request = nullptr;
                     }
 
-                    auto_ptr_char dest(epit->getLocation());
                     SAML2SOAPClient client(soaper, false);
+                    auto_ptr_char dest(epit->getLocation());
                     client.sendSAML(msg.release(), application.getId(), mcc, dest.get());
                     srt.reset(client.receiveSAML());
                     if (!(logoutResponse = dynamic_cast<LogoutResponse*>(srt.get()))) {
                         break;
                     }
                 }
-                catch (std::exception& ex) {
+                catch (const std::exception& ex) {
                     m_log.error("error sending LogoutRequest message: %s", ex.what());
                     soaper.reset();
                 }
@@ -435,9 +443,10 @@ pair<bool,long> SAML2LogoutInitiator::doRequest(
             }
 
             if (session) {
+                time_t revocationExp = session->getExpiration();
                 sessionLocker.assign();
                 session = nullptr;
-                application.getServiceProvider().getSessionCache()->remove(application, httpRequest, &httpResponse);
+                application.getServiceProvider().getSessionCache()->remove(application, httpRequest, &httpResponse, revocationExp);
             }
 
             return ret;
@@ -450,11 +459,11 @@ pair<bool,long> SAML2LogoutInitiator::doRequest(
             application.limitRedirect(httpRequest, returnloc);
             relayState = returnloc;
             httpRequest.absolutize(relayState);
-            cleanRelayState(application, httpRequest, httpResponse);
-            preserveRelayState(application, httpResponse, relayState);
         }
+        cleanRelayState(application, httpRequest, httpResponse);
+        preserveRelayState(application, httpResponse, relayState);
 
-        auto_ptr<LogoutRequest> msg(buildRequest(application, *session, *role, encoder));
+        auto_ptr<LogoutRequest> msg(buildRequest(application, *session, *role, ep->getLocation(), encoder));
         msg->setDestination(ep->getLocation());
 
         // Log the request.
@@ -465,21 +474,22 @@ pair<bool,long> SAML2LogoutInitiator::doRequest(
         }
 
         auto_ptr_char dest(ep->getLocation());
-        ret.second = sendMessage(*encoder, msg.get(), relayState.c_str(), dest.get(), role, application, httpResponse, true);
+        ret.second = sendMessage(*encoder, msg.get(), relayState.c_str(), dest.get(), role, application, httpResponse, "true");
         ret.first = true;
         msg.release();  // freed by encoder
 
         if (session) {
+            time_t revocationExp = session->getExpiration();
             sessionLocker.assign();
             session = nullptr;
-            application.getServiceProvider().getSessionCache()->remove(application, httpRequest, &httpResponse);
+            application.getServiceProvider().getSessionCache()->remove(application, httpRequest, &httpResponse, revocationExp);
         }
     }
-    catch (MetadataException& mex) {
+    catch (const MetadataException& mex) {
         // Less noise for IdPs that don't support logout (i.e. most)
         m_log.info("unable to issue SAML 2.0 logout request: %s", mex.what());
     }
-    catch (std::exception& ex) {
+    catch (const std::exception& ex) {
         m_log.error("error issuing SAML 2.0 logout request: %s", ex.what());
     }
 
@@ -492,8 +502,11 @@ pair<bool,long> SAML2LogoutInitiator::doRequest(
 #ifndef SHIBSP_LITE
 
 auto_ptr<LogoutRequest> SAML2LogoutInitiator::buildRequest(
-    const Application& application, const Session& session, const RoleDescriptor& role, const MessageEncoder* encoder
-    ) const
+    const Application& application,
+    const Session& session,
+    const RoleDescriptor& role,
+    const XMLCh* endpoint,
+    const MessageEncoder* encoder) const
 {
     const PropertySet* relyingParty = application.getRelyingParty(dynamic_cast<EntityDescriptor*>(role.getParent()));
 
@@ -509,23 +522,40 @@ auto_ptr<LogoutRequest> SAML2LogoutInitiator::buildRequest(
     }
 
     const NameID* nameid = session.getNameID();
-    pair<bool,const char*> flag = relyingParty->getString("encryption");
-    if (flag.first &&
-        (!strcmp(flag.second, "true") || (encoder && !strcmp(flag.second, "front")) || (!encoder && !strcmp(flag.second, "back")))) {
-        auto_ptr<EncryptedID> encrypted(EncryptedIDBuilder::buildEncryptedID());
-        MetadataCredentialCriteria mcc(role);
-        encrypted->encrypt(
-            *nameid,
-            *(application.getMetadataProvider()),
-            mcc,
-            encoder ? encoder->isCompact() : false,
-            relyingParty->getXMLString("encryptionAlg").second
+    pair<bool, const char*> flag = getString("encryption");
+    if (!flag.first)
+        flag = relyingParty->getString("encryption");
+    auto_ptr_char dest(endpoint);
+    if (SPConfig::shouldSignOrEncrypt(flag.first ? flag.second : "conditional", dest.get(), encoder != nullptr)) {
+        try {
+            auto_ptr<EncryptedID> encrypted(EncryptedIDBuilder::buildEncryptedID());
+            MetadataCredentialCriteria mcc(role);
+            encrypted->encrypt(
+                *nameid,
+                *(application.getMetadataProvider()),
+                mcc,
+                encoder ? encoder->isCompact() : false,
+                relyingParty->getXMLString("encryptionAlg").second
             );
-        msg->setEncryptedID(encrypted.get());
-        encrypted.release();
+            msg->setEncryptedID(encrypted.get());
+            encrypted.release();
+        }
+        catch (std::exception& ex) {
+            // If we're encrypting deliberately, failure should be fatal.
+            if (flag.first && strcmp(flag.second, "conditional")) {
+                throw;
+            }
+            // If opportunistically, just log and move on.
+            m_log.info("Conditional encryption of NameID in LogoutRequest failed: %s", ex.what());
+            auto_ptr<NameID> namewrapper(nameid->cloneNameID());
+            msg->setNameID(namewrapper.get());
+            namewrapper.release();
+        }
     }
     else {
-        msg->setNameID(nameid->cloneNameID());
+        auto_ptr<NameID> namewrapper(nameid->cloneNameID());
+        msg->setNameID(namewrapper.get());
+        namewrapper.release();
     }
 
     XMLCh* msgid = SAMLConfig::getConfig().generateIdentifier();
