@@ -99,7 +99,7 @@ pair<bool,long> WAYFSessionInitiator::run(SPRequest& request, string& entityID,
     pair<bool,const char*> prop;
     const Handler* ACS = nullptr;
     const Application& app = request.getApplication();
-    pair<bool,const char*> discoveryURL = pair<bool,const char*>(true, m_url);
+    pair<bool,const char*> discoveryURL = pair<bool,const char*>(false, nullptr);
 
     if (isHandler) {
         prop.second = request.getParameter("acsIndex");
@@ -118,9 +118,7 @@ pair<bool,long> WAYFSessionInitiator::run(SPRequest& request, string& entityID,
         recoverRelayState(request.getApplication(), request, request, target, false);
         request.getApplication().limitRedirect(request, target.c_str());
 
-        prop.second = request.getParameter("discoveryURL");
-        if (prop.second && *prop.second)
-            discoveryURL.second = prop.second;
+        discoveryURL = getString("discoveryURL", request, HANDLER_PROPERTY_MAP);
     }
     else {
         // Check for a hardwired target value in the map or handler.
@@ -132,7 +130,11 @@ pair<bool,long> WAYFSessionInitiator::run(SPRequest& request, string& entityID,
 
         discoveryURL = request.getRequestSettings().first->getString("discoveryURL");
     }
-    
+
+    if (!discoveryURL.first)
+        discoveryURL.second = m_url;
+    m_log.debug("sending request to WAYF (%s)", discoveryURL.second);
+
     if (!ACS) {
         // Try fixed index property.
         pair<bool,unsigned int> index = getUnsignedInt("acsIndex", request, HANDLER_PROPERTY_MAP|HANDLER_PROPERTY_FIXED);
@@ -149,10 +151,6 @@ pair<bool,long> WAYFSessionInitiator::run(SPRequest& request, string& entityID,
             throw ConfigurationException("Unable to locate a SAML 1.x ACS endpoint to use for response.");
     }
 
-    if (!discoveryURL.first)
-        discoveryURL.second = m_url;
-    m_log.debug("sending request to WAYF (%s)", discoveryURL.second);
-
     // Since we're not passing by index, we need to fully compute the return URL.
     // Compute the ACS URL. We add the ACS location to the base handlerURL.
     string ACSloc = request.getHandlerURL(target.c_str());
