@@ -33,7 +33,10 @@
 #include <errno.h>
 #include <stack>
 #include <sstream>
+#include <boost/lexical_cast.hpp>
+#include <xercesc/sax/SAXException.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
+#include <xercesc/util/OutOfMemoryException.hpp>
 
 #include <xmltooling/util/NDC.h>
 #include <xmltooling/util/XMLHelper.h>
@@ -49,7 +52,9 @@
 using namespace shibsp;
 using namespace xmltooling;
 using namespace std;
+
 using xercesc::DOMElement;
+using boost::lexical_cast;
 
 namespace shibsp {
 
@@ -60,7 +65,7 @@ namespace shibsp {
         SocketPool(Category& log, const SocketListener* listener)
             : m_log(log), m_listener(listener), m_lock(Mutex::create()) {}
         ~SocketPool();
-        SocketListener::ShibSocket get();
+        SocketListener::ShibSocket get(bool newSocket=false);
         void put(SocketListener::ShibSocket s);
 
     private:
@@ -126,7 +131,7 @@ SocketListener::ShibSocket SocketPool::connect()
     if (!connected) {
         m_log.crit("socket server unavailable, failing");
         m_listener->close(sock);
-        throw ListenerException("Cannot connect to shibd process, a site adminstrator should be notified.");
+        throw ListenerException("Cannot connect to shibd process, a site administrator should be notified that this web server has malfunctioned.");
     }
 
     m_log.debug("socket (%u) connected successfully", sock);
@@ -145,8 +150,11 @@ SocketPool::~SocketPool()
     }
 }
 
-SocketListener::ShibSocket SocketPool::get()
+SocketListener::ShibSocket SocketPool::get(bool newSocket)
 {
+    if (newSocket)
+        return connect();
+
     m_lock->lock();
     if (m_pool.empty()) {
         m_lock->unlock();
@@ -165,7 +173,7 @@ void SocketPool::put(SocketListener::ShibSocket s)
 }
 
 SocketListener::SocketListener(const DOMElement* e)
-    : m_catchAll(false), log(&Category::getInstance(SHIBSP_LOGCAT".Listener")),
+    : m_catchAll(false), log(&Category::getInstance(SHIBSP_LOGCAT ".Listener")),
         m_shutdown(nullptr), m_stackSize(0), m_socket((ShibSocket)0)
 {
     // Are we a client?
@@ -309,7 +317,8 @@ DDF SocketListener::send(const DDF& in)
     int retry = 1;
     SocketListener::ShibSocket sock;
     while (retry >= 0) {
-        sock = m_socketpool->get();
+        // On second time in, just get a new socket.
+        sock = m_socketpool->get(retry == 0);
 
         int outlen = ostr.length();
         len = htonl(outlen);
@@ -373,7 +382,7 @@ DDF SocketListener::send(const DDF& in)
             except=XMLToolingException::fromString(out.string());
             log->error("remoted message returned an error: %s", except->what());
         }
-        catch (XMLToolingException& e) {
+        catch (const XMLToolingException& e) {
             log->error("caught XMLToolingException while building the XMLToolingException: %s", e.what());
             log->error("XML was: %s", out.string());
             throw ListenerException("Remote call failed with an unparsable exception.");
@@ -392,18 +401,25 @@ bool SocketListener::log_error(const char* fn) const
         fn = "unknown";
 #ifdef WIN32
     int rc=WSAGetLastError();
+    if (rc == WSAECONNRESET) {
+        log->debug("socket connection reset");
+        return false;
+    }
 #else
     int rc=errno;
 #endif
+    const char *msg;
 #ifdef HAVE_STRERROR_R
     char buf[256];
-    memset(buf,0,sizeof(buf));
-    strerror_r(rc,buf,sizeof(buf));
-    log->error("socket call (%s) resulted in error (%d): %s",fn, rc, isprint(*buf) ? buf : "no message");
+#ifdef STRERROR_R_CHAR_P
+    msg = strerror_r(rc,buf,sizeof(buf));
 #else
-    const char* buf=strerror(rc);
-    log->error("socket call (%s) resulted in error (%d): %s", fn, rc, isprint(*buf) ? buf : "no message");
+    msg = strerror_r(rc,buf,sizeof(buf)) ? "<translation failed>" : buf;
 #endif
+#else
+    msg=strerror(rc);
+#endif
+    log->error("failed socket call (%s), result (%d): %s", fn, rc, isprint(*msg) ? msg : "no message");
     return false;
 }
 
@@ -429,9 +445,7 @@ ServerThread::ServerThread(SocketListener::ShibSocket& s, SocketListener* listen
     : m_sock(s), m_child(nullptr), m_listener(listener)
 {
 
-    ostringstream buf;
-    buf << "[" << id << "]";
-    m_id = buf.str();
+    m_id = string("[") + lexical_cast<string>(id) + "]";
 
     // Create the child thread
     m_child = Thread::create(server_thread_fn, (void*)this, m_listener->m_stackSize);
@@ -499,7 +513,7 @@ void ServerThread::run()
 
 int ServerThread::job()
 {
-    Category& log = Category::getInstance(SHIBSP_LOGCAT".Listener");
+    Category& log = Category::getInstance(SHIBSP_LOGCAT ".Listener");
 
     bool incomingError = true;  // set false once incoming message is received
     ostringstream sink;
@@ -539,6 +553,12 @@ int ServerThread::job()
         DDFJanitor jin(in);
         is >> in;
 
+        string appid;
+        const char* aid = in["application_id"].string();
+        if (aid)
+            appid = string("[") + aid + "]";
+        NDC ndc(appid);
+
         log.debug("dispatching message (%s)", in.name() ? in.name() : "unnamed");
 
         incomingError = false;
@@ -546,14 +566,50 @@ int ServerThread::job()
         // Dispatch the message.
         m_listener->receive(in, sink);
     }
-    catch (XMLToolingException& e) {
+    catch (const xercesc::DOMException& e) {
+        auto_ptr_char temp(e.getMessage());
+        if (incomingError)
+            log.error("error processing incoming message: %s", temp.get() ? temp.get() : "no message");
+        XMLParserException ex(string("DOM error: ") + (temp.get() ? temp.get() : "no message"));
+        DDF out=DDF("exception").string(ex.toString().c_str());
+        DDFJanitor jout(out);
+        sink << out;
+    }
+    catch (const xercesc::SAXException& e) {
+        auto_ptr_char temp(e.getMessage());
+        if (incomingError)
+            log.error("error processing incoming message: %s", temp.get() ? temp.get() : "no message");
+        XMLParserException ex(string("SAX error: ") + (temp.get() ? temp.get() : "no message"));
+        DDF out=DDF("exception").string(ex.toString().c_str());
+        DDFJanitor jout(out);
+        sink << out;
+    }
+    catch (const xercesc::XMLException& e) {
+        auto_ptr_char temp(e.getMessage());
+        if (incomingError)
+            log.error("error processing incoming message: %s", temp.get() ? temp.get() : "no message");
+        XMLParserException ex(string("Xerces error: ") + (temp.get() ? temp.get() : "no message"));
+        DDF out=DDF("exception").string(ex.toString().c_str());
+        DDFJanitor jout(out);
+        sink << out;
+    }
+    catch (const xercesc::OutOfMemoryException& e) {
+        auto_ptr_char temp(e.getMessage());
+        if (incomingError)
+            log.error("error processing incoming message: %s", temp.get() ? temp.get() : "no message");
+        XMLParserException ex(string("Out of memory error: ") + (temp.get() ? temp.get() : "no message"));
+        DDF out=DDF("exception").string(ex.toString().c_str());
+        DDFJanitor jout(out);
+        sink << out;
+    }
+    catch (const XMLToolingException& e) {
         if (incomingError)
             log.error("error processing incoming message: %s", e.what());
         DDF out=DDF("exception").string(e.toString().c_str());
         DDFJanitor jout(out);
         sink << out;
     }
-    catch (exception& e) {
+    catch (const exception& e) {
         if (incomingError)
             log.error("error processing incoming message: %s", e.what());
         ListenerException ex(e.what());
